/*! \author F.Satzger
  \brief A click sensitive grid widget.

  This widget provides a dynamic grid which is sensitive to clicks (if not disabled).
 */

#ifndef GRID_H
#define GRID_H

#include <QWidget>

#include <QtCore>
#include <algorithm>
#include <QPainter>
#include <QMouseEvent>
#include <cmath>

class Grid : public QWidget
{
		Q_OBJECT
	public:
		//! Constructor
		Grid(QWidget *parent = 0,unsigned int w=1,unsigned int h=1);

		/*! \brief Resizes the grid.
		  \param width
		  \param height
		  \param preserve Preserves current grid state if true. If new grid is bigger than old one,
		  new cells get set to 0. If grid is smaller, grid state is clipped right/bottom.
		*/
		void setGridSize(unsigned int width, unsigned int height, bool preserve=true);
		/*! \brief Sets a complete new grid.
		  The grid pointer should be deleted by the caller.
		  \param width Length of first array dimension
		  \param height Length of second array dimension
		  \param grid New grid state array
		*/
		void setGrid(const char *const * grid, unsigned int width, unsigned int height);
		/*! \brief Sets a cell.
		  \param x
		  \param y
		  \param state [optional] Defaults to 1
		  \return false if (x,y) out of bounds
		 */
		bool setCell(unsigned int x,unsigned int y, char state=1);
		/*! \brief Sets a cell.
		  \see setCell(unsigned int x, unsigned int y, char state);
		 */
		bool setCell(QPoint p, char state=1);
		/*! \brief Clears a cell.
		  Same as setGrid (x, y, 0).
		  \param x
		  \param y
		  \return false if (x,y) out of bounds
		*/
		bool clearCell (unsigned int x, unsigned int y);
		/*! \brief Clears a cell.
		 \see clearCell(unsigned int x, unsigned int y)
		*/
		bool clearCell (QPoint p);
		/*! \brief Clears all cells with a specific state
		  \param state
		  \see clearCell(unsigned int x, unsigned int y)
		*/
		void clearCells(char state);
		/*! \brief Clears grid
		  \see clearCell(unsigned int x, unsigned int y)
		*/
		void clearGrid();
		/*! \brief Returns state of a cell.
		  \see getCell(unsigned int x, unsigned int y) const
		*/
		char getCell(QPoint p) const;
		/*! \brief Returns state of a cell.
		  \param x
		  \param y
		  \return Cell state. If coordinates are out of bounds, 0 is returned.
		*/
		char getCell(unsigned int x, unsigned int y) const;

		/*! \brief Returns complete grid.
		*/
		const char *const * getGrid() const;

		/*! \brief Returns grid size.
		*/
		const QSize getGridSize()const;

		/*! \brief Sets the color for a cell state.
		*/
		void setCellColor(char state,QColor color);

		/*! \brief Sets if autoSet function is activated.
		 * If autoSet is activated, grid cells get set automatically on leftclick and deset on
		 * rightclick.
		 * \param enable autoSet enabled if true
		 * \see getAutoSet
		*/
		void setAutoSet(bool enable);

		/*! \brief Returns if autoSet is enabled
		 * \see setAutoSet
		 */
		bool getAutoSet();

		//! Destructor
		virtual ~Grid();

	protected:
		//! Inherited from QT
		virtual void paintEvent(QPaintEvent *);
		//! Paints basic grid. Gets called by paintEvent().
		virtual void paintGrid(QPainter&);
		//! Paints state of individual grid cells. Gets called by paintEvent().
		virtual void paintGridState(QPainter&);
		//! Inherited from QT
		virtual void mouseReleaseEvent(QMouseEvent *);
		/*! \brief Decodes a position to "cell coordinates"
		  \param p Coordinate (pixels) relative to widget
		  \return Coordinates of cell containing the given point
		*/
		virtual QPoint decodePosition(QPoint p) const;
		/*! \brief Returns size of one cell
		*/
		virtual QSize cellSize() const;

		/*! \brief Number of grid cells in a row */
		unsigned int gridWidth;
		/*! \brief Number of grid cells in a column */
		unsigned int gridHeight;
		/*! \brief Array of the states of Grid#gridWidth times Grid#gridHeight cells */
		char** gridState;


	signals:
		/*! \brief Signals a click to a cell.
		  \param cell Cell user clicked on
		  \param button Mousebutton used
		*/
		void clicked(QPoint cell, Qt::MouseButton button) const;
	public slots:

	protected:
		//! Deletes internal Grid structures
		void deleteGrid();

		//! \brief Colors used for grid states
		QVector<QColor> colors;

		//! \brief Sets cells automatically on mouse click
		bool autoSet;

};

#endif // GRID_H
